// -*- C++ -*-
//    Title: particle.cpp
//  Created: Sun Mar 14 12:00:27 2010
//   Author: Tim Weyrich <t.weyrich@cs.ucl.ac.uk>
//      $Id: $
//
// copyright (c) 2010, University College London
//

#ifdef _WIN32
#  include <windows.h>
#endif
#include <stdio.h>
#include <math.h>

#include <iostream>
#include <deque>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "myclock.h"
#include "amra_vector.h"
#include "random.h"


struct Particle
{
    Particle(const Point &_pos, const Vector &_vel)
        : pos(_pos),
          vel(_vel),
          creationTimeStamp(0)
    { }

    Point  pos;
    Vector  vel;
    // Add any additional attributes here
    // ...

    double  creationTimeStamp;
};

class ParticleSystem
{
  public:
    ParticleSystem()
        : gravity(0,-9.81f,0),
          timeStamp(myclock())
    { }
    
    std::deque<Particle>  particles;

    Vector  gravity;
    double  timeStamp;
};

ParticleSystem  ps;

// Window size
//
const int  WIDTH = 500;
const int  HEIGHT = 500;

// Maximum particle age in seconds
//
const double  MAXAGE = 5;

bool boids = 0;

void simulate(void)
{
    double  now = myclock();

    // Here: simulate particles between the times "ps.timeStamp" and "now"
    // ...

    ps.timeStamp = now;
}


void idle(void)
{
    // Add a particle if we're not at the limit.
    if( boids )
        simulateBoids();
    else
        simulate();
    
    glutPostRedisplay();
}

void display(void)
{
}

void mouse( int button, int state, int x, int y )
{
}

void mouseActiveMotion( int x, int y )
{
}

void keyboard(unsigned char key,int x,int y)
{
}

void glutResize(int w, int h)
{
    if (w!=WIDTH || h!=HEIGHT) 
        glutReshapeWindow(WIDTH,HEIGHT);    // if resized, snap back to original size/shape
    glutPostRedisplay();
}

void create_window()
{
    // create a window with the desired size and title
    glutInitWindowSize(WIDTH,HEIGHT);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
    glutCreateWindow("Particle System");

    // specify the function to be used for rendering in this window
    glutDisplayFunc( display );
    glutIdleFunc( idle );
    glutKeyboardFunc( keyboard );
    glutMouseFunc( mouse );
    glutMotionFunc( mouseActiveMotion );
    glutReshapeFunc( glutResize );

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluOrtho2D(0.0,(double)WIDTH-1,0.0,(double)HEIGHT-1);
    glViewport(0,0,WIDTH,HEIGHT);
}

int main(int argc, char** argv)
{
#if 0
    // demonstrate myclock() function
    std::cout << myclock() << std::endl;
#ifdef _WIN32
    Sleep((DWORD)1.5e3);
#else
    usleep(1.5e6);
#endif
    std::cout << myclock() << std::endl;
    exit(0);
#endif

    glutInit(&argc, argv);
    create_window();

    glutMainLoop();
}
